// document class

#ifndef __DOC_H__
#define __DOC_H__

#include "midi.h"
#include "ctrlmap.h"
#include "keymap.h"



class Doc
{
public:
	// setup fucntions
	Doc();
	~Doc();

	void reset();
	void reset_keys();
	void reset_controllers();


	// file IO
	void load(const string &filename);
	void save(const string &filename);


	// global state
	string mFilename;

	bool mDirty;


	// MIDI objects
	enum { NUM_INS  = 2 };
	enum { NUM_OUTS	= 4 };

	MidiInput *mIn[NUM_INS];
	MidiOutput *mOut[NUM_OUTS];


	// mappings
	bool mKeyActive;
	bool mCtrlActive;

	enum { NUM_KEYS  = 128  };
	enum { NUM_CTRLS = CtrlMap::NUM_TYPES };

	KeyMap *mKeyMap[NUM_KEYS];
	CtrlMap *mCtrlMap[NUM_CTRLS];

	int mCtrlIn[NUM_CTRLS];
	int mCtrlOut[NUM_CTRLS];

	int mLastNote;

	int mThruPort;

	CListBox *mKeyMapCtrl;
	CButton *mKeyActiveCtrl;

	CListBox *mCtrlMapCtrl;
	CButton *mCtrlActiveCtrl;

	void init_controls(CListBox *key_ctrl, CButton *key_active, CListBox *ctrl_ctrl, CButton *ctrl_active);

	void update_key_controls();
	void update_ctrl_controls();


	// the proper MIDI filtering stuff
	void process_note_on(int channel, int key, int vel, int port);
	void process_note_off(int channel, int key, int vel, int port);

	void process_controller(int channel, int ctrl, int value);

	void process(int msg, int channel, int d1, int d2, int port);


	// remember which notes are playing
	class NotesOn
	{
	public:
		NotesOn() { mN = 0; }

		enum { N = KeyMap::MAX_SPLITS * KeyMap::MAX_SPLITS };

		int mPort[N];
		int mKey[N];

		int mN;

		void add(int port, int key)
		{
			if (mN < N) {
				mPort[mN] = port;
				mKey[mN] = key;
				mN++;
			}
		}
	};

	NotesOn mNotesOn[NUM_INS][128];
};




#endif		// __DOC_H__