// MIDI interface class

#ifndef __MIDI_H__
#define __MIDI_H__



// MIDI messages
#define MIDI_NOTE_OFF			0x80
#define MIDI_NOTE_ON			0x90
#define MIDI_KEY_PRESSURE		0xA0
#define MIDI_CONTROLLER			0xB0
#define MIDI_PROGRAM_CHANGE		0xC0
#define MIDI_AFTERTOUCH			0xD0
#define MIDI_PITCH_BEND			0xE0
#define MIDI_SYSTEM				0xF0



class Midi
{
public:
	// setup functions
	Midi(vector<string> &interfaces, class Doc *doc);
	virtual ~Midi();
	
	void init_controls(CComboBox *list_ctrl, CComboBox *channel_ctrl, CButton *enable_ctrl);

	void update_controls();

	void set(int port, bool active);

	void load(FILE *file);
	void save(FILE *file);

	virtual void open() = 0;
	virtual void close() = 0;


	// data
	int mPort;
	int mChannel;
	bool mActive;

	vector<string> &mInterfaces;

	class Doc *mDoc;

	CComboBox *mListCtrl;
	CComboBox *mChannelCtrl;
	CButton *mEnableCtrl;
};



class MidiInput : public Midi
{
public:
	// setup fucntions
	MidiInput(class Doc *doc);
	~MidiInput();

	virtual void open();
	virtual void close();

	void in(int data);

	static void CALLBACK callback(HMIDIIN handle, UINT msg, DWORD instance, DWORD param1, DWORD param2);

	HMIDIIN mHandle;

	int mRecentNote;
	int mRecentController;
};



class MidiOutput : public Midi
{
public:
	// setup fucntions
	MidiOutput(class Doc *doc);
	~MidiOutput();

	virtual void open();
	virtual void close();

	void out(int data);

	void panic();

	HMIDIOUT mHandle;
};




#endif		// __MIDI_H__
